/////////////////////////////////////////////////////////////////////////////
//                   SOFTWARE COPYRIGHT NOTICE AGREEMENT                   //
//       This software and its documentation are copyright (2008) by the   //
//   Broad Institute/Massachusetts Institute of Technology.  All rights    //
//   are reserved.  This software is supplied without any warranty or      //
//   guaranteed support whatsoever. Neither the Broad Institute nor MIT    //
//   can be responsible for its use, misuse, or functionality.             //
/////////////////////////////////////////////////////////////////////////////
/*
 * \file ArtifactualSequenceDetector.h
 * \author tsharpe
 * \date Apr 30, 2009
 *
 * \brief Detects sequences that match a DB of artifactual sequences.
 */
#ifndef MICRO_ARTIFACTUALSEQUENCEDETECTOR_H_
#define MICRO_ARTIFACTUALSEQUENCEDETECTOR_H_

#include "Basevector.h"
#include "pairwise_aligners/SuffixTree.h"

class ArtifactualSequenceDetector
{
public:
    /// create one from a set of artifactual sequences to match
    ArtifactualSequenceDetector( vecbvec const& artifacts = ArtifactualSequenceDetector::getDefaultSolexaArtifacts() );

    /// create one from a fastb file
    /// if the file name is null or empty, the standard Solexa sequences will be used
    ArtifactualSequenceDetector( char const* );

    /// test a read (or other sequence) for matches against artifacts (or their rev comps)
    bool isArtifactual( bvec const& read ) const
    { return isArtifactual(read,DEFAULT_MIN_MATCH_LEN); }

    /// test a read (or other sequence) for exact matches against artifacts (or rev comps) of a specified minimum length
    bool isArtifactual( bvec const& read, unsigned int minMatchLen ) const
    { return mArtifacts.matches(read,minMatchLen); }

    /// all the standard Solexa adatpers, PCR primers, and sequencing primers
    static vecbvec const& getDefaultSolexaArtifacts();

private:
    SuffixTree mArtifacts;
    static unsigned int const DEFAULT_MIN_MATCH_LEN = 14;
};

#endif /* MICRO_ARTIFACTUALSEQUENCEDETECTOR_H_ */
