/////////////////////////////////////////////////////////////////////////////
//                   SOFTWARE COPYRIGHT NOTICE AGREEMENT                   //
//       This software and its documentation are copyright (2008) by the   //
//   Broad Institute/Massachusetts Institute of Technology.  All rights    //
//   are reserved.  This software is supplied without any warranty or      //
//   guaranteed support whatsoever. Neither the Broad Institute nor MIT    //
//   can be responsible for its use, misuse, or functionality.             //
/////////////////////////////////////////////////////////////////////////////
/*
 * \file VectorTrimmer.cc
 * \author tsharpe
 * \date Apr 29, 2009
 *
 * \brief Class to trim adapter sequence from read ends.
 */
#include "micro/VectorTrimmer.h"

String VectorTrimmer::gDefaultSolexaSequence("AGATCGGAAGAGCTCGTATGCCGTCTTCTGCTTGAAAAAAAAAAAAAAAAAAAAAAAAA");

uint VectorTrimmer::trimmedLength( bvec const& read )
{
    uint readLen = read.size();
    uint vecLen = min(mVecSeq.size(),readLen) + 1;

    while ( vecLen-- > 0 ) // nibble one base off the end of the vector sequence each time through
    {
        // brute-force match the beginning of the vector sequence to the end of the read
        uint nErrs = 0;
        for ( uint vecIdx = 0, readIdx = readLen - vecLen; vecIdx < vecLen; ++vecIdx, ++readIdx )
        {
            if ( vecIdx == 10 && 4*nErrs <= 10 ) // trim if there are fewer than 2 mismatches in the first 10 bases
                return readLen-vecLen; // EARLY RETURN!

            if ( read[readIdx] != mVecSeq[vecIdx] )
                nErrs += 1;
        }
        if ( 4*nErrs <= vecLen ) // trim if 3/4 of the bases match
            return readLen - vecLen; // EARLY RETURN!
    }

    // we never actually reach here
    // on the last trip through when vecLen==0 we always trim nothing (because 4*0 <= 0)
    return readLen;
}

uint VectorTrimmer::exactMatchLength( bvec const& read )
{
    uint readLen = read.size();
    uint vecLen = min(mVecSeq.size(),readLen) + 1;

    while ( vecLen-- > 0 ) // nibble one base off the end of the vector sequence each time through
    {
        // brute-force match the beginning of the vector sequence to the end of the read
        bool isExact = true;
        for ( uint vecIdx = 0, readIdx = readLen - vecLen; isExact && vecIdx < vecLen; ++vecIdx, ++readIdx )
        {
            if ( read[readIdx] != mVecSeq[vecIdx] )
                isExact = false;
        }
        if ( isExact )
            return vecLen; // EARLY RETURN!
    }

    // we never actually reach here
    // on the last trip through when vecLen==0 we always get an exact match (of 0 bases)
    return 0;
}
