/////////////////////////////////////////////////////////////////////////////
//                   SOFTWARE COPYRIGHT NOTICE AGREEMENT                   //
//       This software and its documentation are copyright (2008) by the   //
//   Broad Institute/Massachusetts Institute of Technology.  All rights    //
//   are reserved.  This software is supplied without any warranty or      //
//   guaranteed support whatsoever. Neither the Broad Institute nor MIT    //
//   can be responsible for its use, misuse, or functionality.             //
/////////////////////////////////////////////////////////////////////////////
/*
 * \file VectorTrimmer.h
 * \author tsharpe
 * \date Apr 29, 2009
 *
 * \brief Class to trim adapter sequence from read ends.
 */
#ifndef MICRO_VECTORTRIMMER_H_
#define MICRO_VECTORTRIMMER_H_

#include "Basevector.h"
#include "String.h"
#include "Vec.h"

class VectorTrimmer
{
public:
    /// create a trimmer that uses the default solexa trim sequence
    VectorTrimmer()
    : mVecSeq(getDefaultSolexaSequence())
    {}

    VectorTrimmer( String const& vectorSequence )
    : mVecSeq(vectorSequence)
    {}

    /// create a trimmer that uses the specified trim sequence
    VectorTrimmer( bvec const& vectorSequence )
    : mVecSeq(vectorSequence)
    {}

    // compiler-supplied copying and destructor are OK

    /// the length of the read post-trimming
    unsigned int trimmedLength( bvec const& readSequence );

    /// the max length of an exact match between end of read and beginning of vector
    unsigned int exactMatchLength( bvec const& readSequence );

    /// does the read contain vector sequence?
    bool isVecLike( bvec const& readSequence )
    { return trimmedLength(readSequence) < readSequence.size(); }

    /// set the vec<Bool> for each read if it's vec-like
    void mark( vecbasevector const& reads, vec<Bool>& veclike )
    { int len = reads.size();
      veclike.resize(len);
      for ( int iii = 0; iii < len; ++iii )
          veclike[iii] = isVecLike(reads[iii]); }

    void trim( bvec& readSequence )
    { readSequence.resize(trimmedLength(readSequence)); }

    void trim( vecbasevector& reads )
    { int len = reads.size();
      for ( int iii = 0; iii < len; ++iii )
          trim(reads[iii]); }

    static String const& getDefaultSolexaSequence()
    { return gDefaultSolexaSequence; }

private:
    bvec mVecSeq;
    static String gDefaultSolexaSequence;
};

#endif /* MICRO_VECTORTRIMMER_H_ */
