/////////////////////////////////////////////////////////////////////////////
//                   SOFTWARE COPYRIGHT NOTICE AGREEMENT                   //
//       This software and its documentation are copyright (2008) by the   //
//   Broad Institute/Massachusetts Institute of Technology.  All rights    //
//   are reserved.  This software is supplied without any warranty or      //
//   guaranteed support whatsoever. Neither the Broad Institute nor MIT    //
//   can be responsible for its use, misuse, or functionality.             //
/////////////////////////////////////////////////////////////////////////////

// VAAL4.  Read results of VAAL{1,2,3}, compute polymorphisms versus
// reference.
//
// See VAAL1.cc for general info.

// MakeDepend: dependency VAALCompareCompound

// Convention in output: if sample= ref=<stuff>, then this is a deletion
//   - relative to the reference

#include <limits.h>

#include "MainTools.h"
#include "polymorphism/TruePolyCore.h"
#include "polymorphism/VAALTools.h"
#include "solexa/SolexaTools.h"

int main( int argc, char *argv[] )
{
     RunTime( );

     BeginCommandArguments;
     CommandArgument_String(HEAD);
     CommandArgument_String(REF);
     CommandArgument_String_OrDefault(REF_TRUSTED, "" );
     CommandArgument_String_OrDefault( RUN, "" );
     CommandArgument_Int_OrDefault(TRUNC, 0);
     CommandArgument_String_OrDefault(DIR, "");
     CommandArgument_String_OrDefault(POST, "");
     CommandArgument_String_OrDefault(TRUE_REF, "");
     CommandArgument_String_OrDefault(ASSEMBLY_SUFFIX, "");
     CommandArgument_String_OrDefault(TRUSTED_SUFFIX, "");
     CommandArgument_Int_OrDefault(K, 20);
     CommandArgument_String_OrDefault(CLASSES_TO_PRINT, "{1,2,3,4}");
     CommandArgument_String_OrDefault(PIPELINE, "");
     CommandArgument_Bool_OrDefault(COMPOUND, True);
     CommandArgument_String_OrDefault(OUT_SUFFIX, "");
     EndCommandArguments;
     if ( PIPELINE == "" ) PIPELINE = Getenv("SOLEXA_PIPELINE_DIR");

    // Define directories, etc.

     String workdir, workhead;
     VAALSetup( RUN, HEAD, REF, DIR, POST, workdir, workhead, TRUNC, PIPELINE );
     String KS = ToString(K);
     String outfile = workhead + ".VAAL4.K" + KS + OUT_SUFFIX + ".out";
     cout << "writing to " << outfile << "\n" << endl;
     Ofstream( out, outfile );
     String binary_outfile = workhead + ".VAAL4.K" + KS + OUT_SUFFIX + ".binary";

     String compound_file;
     if (COMPOUND)
          compound_file = workhead + ".VAAL4.K" + KS + OUT_SUFFIX + ".compound";

     // Load data.

     String ASSEMBLY = workhead;
     if ( ASSEMBLY_SUFFIX != "" ) ASSEMBLY += "." + ASSEMBLY_SUFFIX;
     ASSEMBLY += ".assembly.fastb";
     String TRUSTED = workhead;
     if ( ASSEMBLY_SUFFIX != "" ) TRUSTED += "." + ASSEMBLY_SUFFIX;
     TRUSTED += ".assembly.trusted" + TRUSTED_SUFFIX;
     if ( TRUE_REF != "" && !TRUE_REF.Contains( ".fasta", -1 ) )
          TRUE_REF += ".fasta";
     vec<String> HEADS;
     ExpandHead( HEAD, HEADS, TRUNC, PIPELINE );

     String REFX = REF;
     String run = HEADS[0].RevBefore( "/" );
     if ( IsRegularFile( run + "/" + REF ) ) REFX = run + "/" + REF;
     else if ( run.Contains( "/" ) )
     {    run = run.RevBefore( "/" );
          if ( IsRegularFile( run + "/" + REF ) ) REFX = run + "/" + REF;    }

     // If a .trusted vecbitvector for the reference is either explicitly specified
     // by the user or exists alongside the given reference fasta, pass it to TruePolyCore.
     String REFX_TRUSTED = REF_TRUSTED;
     String ref_trusted_default_filename = REFX.RevBefore(".fasta") + ".trusted";
     if ( REFX_TRUSTED == "" && IsRegularFile( ref_trusted_default_filename ) )
       REFX_TRUSTED = ref_trusted_default_filename;
     if( REFX_TRUSTED != "" )
       out << "Applying " << REFX_TRUSTED
           << " to reference when searching for polymorphisms." << endl;

     String TRUE_REFX;
     if ( TRUE_REF != "" )
     {    TRUE_REFX = TRUE_REF;
          String run = HEADS[0].RevBefore( "/" );
          if ( IsRegularFile( run + "/" + TRUE_REF ) )
               TRUE_REFX = run + "/" + TRUE_REF;
          else if ( run.Contains( "/" ) )
          {    run = run.RevBefore( "/" );
               if ( IsRegularFile( run + "/" + TRUE_REF ) )
                    TRUE_REFX = run + "/" + TRUE_REF;    }    }

     vecbitvector nonpolymorphic_bases;
     if ( TRUE_REF == "" )
     {    if ( !COMPOUND )
          {    TruePolyCore( out, ASSEMBLY, REFX, TRUSTED, REFX_TRUSTED, K,
                    CLASSES_TO_PRINT, binary_outfile, "", 0,
                    &nonpolymorphic_bases );    }
          else
          {    TruePolyCore( out, ASSEMBLY, REFX, TRUSTED, REFX_TRUSTED, K,
                    CLASSES_TO_PRINT, binary_outfile, compound_file, 0,
                    &nonpolymorphic_bases );    }
          nonpolymorphic_bases.WriteAll( workhead + ".VAAL4.K" + KS
               + OUT_SUFFIX + ".nonpolymorphic.bitvector" );
          return(0);    }

     temp_file P1( "/tmp/VAAL4.1.XXXXXXX" );
     temp_file P2( "/tmp/VAAL4.2.XXXXXXX" );
     {    Ofstream( out1, P1 );
          if ( !COMPOUND )
          {    TruePolyCore( out1, ASSEMBLY, REFX, TRUSTED, REFX_TRUSTED, K,
                    CLASSES_TO_PRINT, binary_outfile, "", 0,
                    &nonpolymorphic_bases );    }
          else
          {    TruePolyCore( out1, ASSEMBLY, REFX, TRUSTED, REFX_TRUSTED, K, CLASSES_TO_PRINT,
                    binary_outfile, compound_file + ".assembly", 0,
                    &nonpolymorphic_bases );    }
          Ofstream( out2, P2 );
          if ( !COMPOUND )
               TruePolyCore( out2, TRUE_REFX, REFX, "", REFX_TRUSTED, K );
          else
          {    TruePolyCore( out2, TRUE_REFX, REFX, "", REFX_TRUSTED, K, CLASSES_TO_PRINT,
                    "", compound_file + ".truepoly", 0,
                    &nonpolymorphic_bases );    }    }
     nonpolymorphic_bases.WriteAll(
          workhead + ".VAAL4.K" + KS + OUT_SUFFIX + ".nonpolymorphic.bitvector" );

     if ( COMPOUND ) {
        System("VAALCompareCompound COMPOUND_TRUEPOLY=" + compound_file
             + ".truepoly COMPOUND_ASSEMBLY=" + compound_file
             + ".assembly EVENTS_FILE=" + compound_file + ".events SUBEVENTS_FILE="
             + compound_file + ".subevents");
     }
     vec<String> poly1, poly2, false_poly;
     vec<String> summary2;
     Ifstream( in1, P1 );
     Ifstream( in2, P2 );
     String line;
     while(1)
     {    getline( in1, line );
          if ( !in1 ) break;
          out << line << "\n";
          if ( !line.Contains( "#", 0 ) ) poly1.push_back(line);    }
     while(1)
     {    getline( in2, line );
          if ( !in2 ) break;
          if ( line.Contains( "#", 0 ) ) summary2.push_back(line);
          else poly2.push_back(line);    }
     out << "\n";
     Sort(poly2);
     for ( int j = 0; j < poly1.isize( ); j++ )
          if ( !BinMember( poly2, poly1[j] ) ) false_poly.push_back( poly1[j] );
     if ( false_poly.empty( ) ) out << "There are no false polymorphisms.\n";
     else
     {    out << "False polymorphisms:\n\n";
          for ( int j = 0; j < false_poly.isize( ); j++ )
               out << false_poly[j] << "\n";    }
     out << "\nSummary of true polymorphisms:\n\n";
     for ( int j = 0; j < summary2.isize( ); j++ )
     {    if ( summary2[j].size( ) >= 2 && isdigit( summary2[j][1] ) )
               out << summary2[j] << "\n";    }    }
