/////////////////////////////////////////////////////////////////////////////
//                   SOFTWARE COPYRIGHT NOTICE AGREEMENT                   //
//       This software and its documentation are copyright (2008) by the   //
//   Broad Institute/Massachusetts Institute of Technology.  All rights    //
//   are reserved.  This software is supplied without any warranty or      //
//   guaranteed support whatsoever. Neither the Broad Institute nor MIT    //
//   can be responsible for its use, misuse, or functionality.             //
/////////////////////////////////////////////////////////////////////////////

// This is the main executable for VAAL.  See VAAL_manual.doc for a guide as
// to how to use it.

// MakeDepend: dependency MakeLookupTable
// MakeDepend: dependency ShortQueryLookup
// MakeDepend: dependency Qualb
// MakeDepend: dependency Fastb
// MakeDepend: dependency VAAL

#include "MainTools.h"
#include "system/System.h"
#include "solexa/SolexaTools.h"

int main( int argc, char *argv[] )
{
     BeginCommandArguments;
     CommandArgument_String_Doc(SOURCE, "Will look for SOURCE.fasta and "
          "SOURCE.qual, the reads for the sample genome.  "
          "There should be no dots in SOURCE.");
     CommandArgument_String_Doc(REF, "Look for reference file REF.fasta.");
     CommandArgument_String_OrDefault_Doc(REF_TRUSTED, "",
          "A vecbitvector indicating which of the bases in the reference can be trusted.");
     CommandArgument_String_OrDefault_Doc(TRUE_REF, "", 
          "Look for sample genome reference file TRUE_REF.fasta.");
     CommandArgument_String_OrDefault_Doc(RUN, Getenv("LOGNAME"), 
          "Name of subdirectory of SOURCE to put run in.");
     CommandArgument_Int_OrDefault_Doc(TRUNC, 0, 
          "Truncate runs to this many bases.");
     CommandArgument_String_Doc(VECSEQ,
          "vector sequence to be trimmed from right side of reads");
     EndCommandArguments;

     String thePipeline = SOURCE.RevBefore("/"), theFile = SOURCE.RevAfter("/");
     String theDir = (TRUNC != 0) ? thePipeline + "/trunc_" + ToString(TRUNC) : thePipeline;

     if( !IsDirectory( theDir ) ) {
       cerr << "VAALrun looks for input data in the directory "
            << theDir
            << " when called with the given parameters.  Please move input files to this directory."
            << endl;
       exit(1);
     }

     SystemSucceed("MakeLookupTable SOURCE=" + theDir + "/" + REF 
          + ".fasta OUT_HEAD=" + theDir + "/" + REF);
     if ( TRUE_REF != "" )
     {    SystemSucceed("MakeLookupTable SOURCE=" + theDir + "/" + TRUE_REF 
               + ".fasta OUT_HEAD=" + theDir + "/" + TRUE_REF);    }
     SystemSucceed("Qualb PRE=" + theDir + " FILE=" + theFile + ".qual");
     SystemSucceed("Fastb PRE=" + theDir + " FILE=" + theFile + ".fasta");
     SystemSucceed("ShortQueryLookup SEQS=" + theDir + "/" + SOURCE + ".fastb L=" 
          + theDir + "/" + REF + ".lookup OUT_SUFFIX=.qltout");
     SystemSucceed("VAAL HEAD=" + theFile + " REF=" + REF + " REF_TRUSTED=" + REF_TRUSTED
          + " TRUE_REF=" + TRUE_REF + " TRUNC=" + ToString(TRUNC) + " PIPELINE=" + thePipeline
          + " RUN=" + RUN + " VECSEQ=" + VECSEQ);    }
